package chess.services;

import chess.api.models.LoginModel;
import com.google.gson.Gson;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;

/**
 * Contains information about available engines.
 */
@Service
public class EngineInfoService {

    private static final Gson GSON = new Gson();

    /**
     * maps engine names to engine models.
     */
    private HashMap<String, EngineInfoService.EngineModel> engines = new HashMap<>();

    /**
     * Loads engine info from file under given {@code path}
     */
    public EngineInfoService(String path) {
        File file = new File(path);
        if (file.exists()) {
            try {
                readConfigFile(file);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Reads contents of {@code file} and tries to save its contents to {@link #engines} map.
     * @param file
     * @throws IOException
     */
    private void readConfigFile(File file) throws IOException {
        FileInputStream fis;
        try {
            fis = new FileInputStream(file);
            byte data[] = new byte[(int) file.length()];
            fis.read(data);
            fis.close();
            String string = new String(data, "UTF-8");
            EngineModel models[] = GSON.getAdapter(EngineModel[].class).fromJson(string);

            printEngines(models);
            for (EngineModel m : models) {
                engines.put(m.name, m);
            }
        } catch (IOException e) {
            e.printStackTrace();
            throw new IOException("File " + file.getAbsolutePath() + " is corrupted", e);
        }
    }

    /**
     * @return available engines
     */
    public Collection<EngineModel> getAvailableEngines() {
        return engines.values();
    }

    /**
     * Retrieves detailed information about given engine
     * @param name engine name
     * @return information about engine with given {@code name} or null if engine was not found.
     */
    public EngineModel getEngineInfo(String name) {
        return engines.get(name);
    }

    private static void printEngines(EngineModel[] engines) {
        StringBuilder builder = new StringBuilder();
        builder.append("==============================\n");
        builder.append("Available engines:\n");
        for (EngineModel engine : engines) {
            builder.append(engine.toString());
            builder.append("\n");
        }
        builder.append("==============================\n");
        System.out.println(builder.toString());
    }

    public class EngineModel {

        public String name;
        public String path;
        public String description;

        @Override
        public String toString() {
            return "[" + name + ", " + path + ", " + description + "]";
        }
    }
}
