package chess.services;


import chess.api.models.LoginModel;
import com.google.gson.Gson;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.HashMap;

/**
 * Contains information about users.
 */
@Service
public class UserInfoService {

    private static final Gson GSON = new Gson();

    /**
     * maps user's logins to password hash.
     */
    private HashMap<String, String> usersCredentials;

    /**
     * Loads user credentials from file under {@code path}. This file should be in JSON format, containing array of
     * {@link LoginModel} objects.
     */
    public UserInfoService(String path) {
        usersCredentials = new HashMap<>();
        File file = new File(path);
        if (file.exists()) {
            try {
                readConfigFile(file);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Reads contents of {@code file} and stores them in {@link #usersCredentials} map.
     * @param file file with user credentials.
     * @throws IOException
     */
    private void readConfigFile(File file) throws IOException {
        FileInputStream fis;
        try {
            fis = new FileInputStream(file);
            byte data[] = new byte[(int) file.length()];
            fis.read(data);
            fis.close();
            String string = new String(data, "UTF-8");
            LoginModel models[] = GSON.getAdapter(LoginModel[].class).fromJson(string);

            for (LoginModel m : models) {
                usersCredentials.put(m.login, m.password);
            }
            printUserInfo(models);
        } catch (IOException e) {
            e.printStackTrace();
            throw new IOException("File " + file.getAbsolutePath() + " is corrupted", e);
        }
    }

    private static void printUserInfo(LoginModel[] users) {
        StringBuilder builder = new StringBuilder();
        builder.append("==============================\n");
        builder.append("Available users:\n");
        for (LoginModel user : users) {
            builder.append(user.login);
            builder.append("\n");
        }
        builder.append("==============================\n");
        System.out.println(builder.toString());
    }

    /**
     * Checks if given user exists and whether given password hash matches stored hash.
     * @param model contains user login and password.
     * @return true if user was found and password hashes are equal, false otherwise.
     */
    public boolean loginUser(LoginModel model) {
        String password = usersCredentials.get(model.login);
        return password != null && password.equals(DigestUtils.sha256Hex(model.password));
    }
}
